@@ -11,6 +11,11 @@ from mx.Log import *
 #import TraceCalls; TraceCalls.install()
 #log.setup(ignore_level=SYSTEM_LOG_EVERYTHING)
 
+### Globals
+
+# Write debug info
+_debug = 0
+
 ### Constants
 
 # Special keys
@@ -72,15 +77,33 @@ class BeeBaseDict:
         to write the cache data to disk storage.
         
     """
-    name = 'NoName'                             # Name of the dict
-    index = None                                # Index object
-    storage = None                              # Storage object
-    closed = 0                                  # Are we closed ?
-    cache = None                                # Cache dict
-    readonly = 0                                # Are we read-only ?
-    autocommit = 0                              # Autocommit ? (this will
-                                                # perform a commit whenever
-                                                # the cache gets to full)
+    # Name of the dictionary
+    name = 'NoName'
+
+    # Name of the storage file; set in .__init__()
+    storage_name = name + '.dat'
+
+    # Name of the index file; set in .__init__()
+    index_name = name + '.idx'
+
+    # Bee*Index object
+    index = None
+
+    # Bee*Storage object
+    storage = None
+
+    # Is the dictionary closed ?
+    closed = 0
+
+    # Cache dictionary
+    cache = None
+
+    # Is the dictionary read-only ?
+    readonly = 0
+
+    # Run in auto-commit mode ? This will perform a commit whenever
+    # the cache gets to full.
+    autocommit = 0                              
 
     # Max. cache size in number of items to store in the in-memory
     # transaction cache
@@ -132,6 +155,8 @@ class BeeBaseDict:
         """
         # Init instance vars
         self.name = name
+        self.storage_name = name + '.dat'
+        self.index_name = name + '.idx'
         self.cache = {}
         if maxcachesize is not None:
             self.maxcachesize = maxcachesize
@@ -140,14 +165,14 @@ class BeeBaseDict:
 
         # Open storage
         self.storage = BeeStorage.BeeKeyValueStorage(
-                                name + '.dat',
-                                lock= not readonly,
-                                cache=0,
-                                min_recordsize=min_recordsize,
-                                readonly=readonly,
-                                recover=recover)
+            self.storage_name,
+            lock= not readonly,
+            cache=0,
+            min_recordsize=min_recordsize,
+            readonly=readonly,
+            recover=recover)
         
-        # Determine the filemode
+        # Determine the filemode for the index
         if readonly:
             # Readonly mode
             filemode = 1
@@ -156,30 +181,36 @@ class BeeBaseDict:
                 # Create a new file (overwriting a possibly existing one)
                 filemode = 2
             else:
-                # Open a file, create is non-existent
-                filemode = 3
+                # Open an existing file, recreate if non-existing
+                filemode = 0
                 
         # Open the index
         try:
-            
-            if index is BeeIndex.BeeStringIndex:
+
+            # Special argument treatment for the indexes; we check
+            # depending on the used constructor function
+            if (index is BeeIndex.BeeStringIndex or
+                index is BeeIndex.BeeFixedLengthStringIndex):
                 # Calculate the right sectorsize
-                if keysize <= 30:
-                    sectorsize = 256
-                elif keysize <= 73:
-                    sectorsize = 512
-                elif keysize <= 158:
-                    sectorsize = 1024
-                else:
-                    raise IndexError, 'keysize %i is too large' % keysize
-                self.index = index(name + '.idx',
+                sectorsize = self._calc_sectorsize(keysize)
+                print ('Using keysize=%i with sectorsize=%i' %
+                       (keysize, sectorsize))
+                self.index = index(self.index_name,
                                    keysize,
                                    dupkeys=0,
                                    filemode=filemode,
                                    sectorsize=sectorsize)
                 
             elif index is BeeIndex.BeeIntegerIndex:
-                self.index = index(name + '.idx',
+                # keysize is sizeof(long)
+                self.index = index(self.index_name,
+                                   dupkeys=1,
+                                   filemode=filemode,
+                                   sectorsize=256)
+
+            elif index is BeeIndex.BeeFloatIndex:
+                # keysize is sizeof(double)
+                self.index = index(self.index_name,
                                    dupkeys=1,
                                    filemode=filemode,
                                    sectorsize=256)
@@ -187,7 +218,7 @@ class BeeBaseDict:
             else:
                 raise IndexError, 'unknown index type: %s' % repr(index)
 
-        except IOError, why:
+        except (IOError, IndexError), why:
             # Make sure the storage lock is freed
             self.close()
             raise RecreateIndexError,\
@@ -205,14 +236,46 @@ class BeeBaseDict:
         """ Calculate the sectorsize given the keysize.
         
         """
-        if keysize <= 30:
-            return 256
-        elif keysize <= 73:
-            return 512
-        elif keysize <= 158:
-            return 1024
+        # These values were determined using the helper
+        # mxBeeBase/calc-sectorsize.py
+        if BeeIndex.sizeof_bRecAddr == 4:
+            # These figures are good for 32-bit platforms and were
+            # used for mxBeeBase <= 3.2.6 for all platforms, failing
+            # for some keysizes when used on 64-bit platforms. Tested
+            # on Linux x86, Windows x86 and x64.
+            if keysize <= 30:
+                sectorsize = 256
+            elif keysize <= 73:
+                sectorsize = 512
+            elif keysize <= 158:
+                sectorsize = 1024
+            elif keysize <= 329:
+                sectorsize = 2048
+            elif keysize <= 670:
+                sectorsize = 4096
+            else:
+                raise IndexError, 'keysize %i is too large' % keysize
+
+        elif BeeIndex.sizeof_bRecAddr == 8:
+            # 64-bit platforms need different sector sizes
+            if keysize <= 19:
+                sectorsize = 256
+            elif keysize <= 61:
+                sectorsize = 512
+            elif keysize <= 147:
+                sectorsize = 1024
+            elif keysize <= 317:
+                sectorsize = 2048
+            elif keysize <= 659:
+                sectorsize = 4096
+            else:
+                raise IndexError, 'keysize %i is too large' % keysize
+            
         else:
-            raise IndexError, 'keysize %i is too large' % keysize
+            raise IndexError('incompatible platform: sizeof_bRecAddr=%i' %
+                             BeeIndex.sizeof_bRecAddr)
+
+        return sectorsize
 
     def remove_files(self):
 
@@ -225,8 +288,8 @@ class BeeBaseDict:
 
         """
         self.close()
-        os.remove(self.name + '.dat')
-        os.remove(self.name + '.idx')
+        os.remove(self.storage_name)
+        os.remove(self.index_name)
 
     def __len__(self):
 
@@ -273,6 +336,9 @@ class BeeBaseDict:
 
         """ Flush buffers to disk.
         """
+        if self.closed:
+            # Nothing much to do
+            return
         if self.storage is not None:
             self.storage.flush()
         if self.index is not None:
@@ -316,7 +382,7 @@ class BeeBaseDict:
         # Clear cache
         if self.cache is not None:
             self.cache.clear()
-                
+
         # Flush storage and index
         self.flush()
 
@@ -978,9 +1044,6 @@ class BeeDict(BeeBaseDict):
 
         """ Commit all changes and start a new transaction.
         """
-        if __debug__:
-            log(SYSTEM_DEBUG,'Committing all changes for "%s"...',self.name)
-
         # Write all changed entries in the cache to disk
         index = self.index
         index_cursor = index.cursor
@@ -992,12 +1055,15 @@ class BeeDict(BeeBaseDict):
         storage_delete = storage.delete
         readonly = self.readonly
         NotFound = None
+        debug = _debug
+        if debug:
+            log(SYSTEM_DEBUG,'Committing all changes for "%s"...',self.name)
 
         for key,(state,value) in self.cache.items():
 
             if state == MODIFIED:
-                if __debug__:
-                    log.object(SYSTEM_DEBUG,' writing key "%s":' % repr(key), value)
+                if debug:
+                    log.object(SYSTEM_DEBUG,' writing key %r:' % repr(key), value)
                 if readonly:
                     raise Error,'dict is read-only'
                 hashvalue = hash(key)
@@ -1026,8 +1092,8 @@ class BeeDict(BeeBaseDict):
                     index[hashvalue] = address
 
             elif state == DELETED:
-                if __debug__:
-                    log(SYSTEM_DEBUG,' deleting key "%s"',key)
+                if debug:
+                    log(SYSTEM_DEBUG,' deleting key %r', repr(key))
                 if readonly:
                     raise Error,'dict is read-only'
                 hashvalue = hash(key)
@@ -1242,6 +1308,8 @@ class BeeStringDict(BeeBaseDict):
 
     """ A dictionary that is indexed by limited size strings.
 
+        The key strings may *not* contain embedded \0 characters.
+
         Though the keys must be strings of limited size, the values
         can be any pickleable object.
 
@@ -1294,9 +1362,6 @@ class BeeStringDict(BeeBaseDict):
 
         """ Commit all changes and start a new transaction.
         """
-        if __debug__:
-            log(SYSTEM_DEBUG,'Committing all changes for "%s"...',self.name)
-
         # Write all changed entries in the cache to disk
         index = self.index
         index_get = index.get
@@ -1305,12 +1370,15 @@ class BeeStringDict(BeeBaseDict):
         storage_delete = storage.delete
         readonly = self.readonly
         NotFound = None
+        debug = _debug
+        if debug:
+            log(SYSTEM_DEBUG,'Committing all changes for "%s"...',self.name)
 
         for key,(state,value) in self.cache.items():
 
             if state == MODIFIED:
-                if __debug__:
-                    log.object(SYSTEM_DEBUG,' writing key "%s":' % repr(key), value)
+                if debug:
+                    log.object(SYSTEM_DEBUG,' writing key %s:' % repr(key), value)
                 address = index_get(key,NotFound)
                 if address is not NotFound:
                     # Update storage
@@ -1326,8 +1394,8 @@ class BeeStringDict(BeeBaseDict):
                     index[key] = address
 
             elif state == DELETED:
-                if __debug__:
-                    log(SYSTEM_DEBUG,' deleting key "%s"',key)
+                if debug:
+                    log(SYSTEM_DEBUG,' deleting key %s', repr(key))
                 if readonly:
                     raise Error,'dict is read-only'
                 address = index_get(key,NotFound)
@@ -1457,9 +1525,65 @@ freeze(BeeStringDict)
 
 ###
 
+class BeeFixedLengthStringDict(BeeStringDict):
+
+    """ A dictionary that is indexed by fixed length strings.
+
+        The key strings may contain embedded \0 characters.
+
+        Though the keys must be strings of limited size, the values
+        can be any pickleable object.
+
+    """
+    def __init__(self,name,keysize=10,min_recordsize=0,readonly=0,recover=0,
+                 autocommit=0,validate=0, maxcachesize=None,
+
+                 basemethod=BeeBaseDict.__init__):
+
+        """ Create an instance using name as dictionary name.
+
+            Two files will be created: <name>.dat and <name>.idx.
+
+            keysize gives the maximal size of the strings used as
+            index keys.
+
+            min_recordsize gives a hint to the expected typical size
+            of (key,value) pickles: all records will have at least
+            this size.
+
+            To open the dictionary in recovery mode, pass a keyword
+            recover=1. Then run .recover() and reopen using the normal
+            settings.
+            
+            If autocommit is true the cache control will do an
+            automatic .commit() whenever the transaction log
+            overflows.
+
+            If validate is true, the dictionary will run a validation
+            check after having successfully opened storage and index.
+            RecreateIndexError or RecoverError exceptions could be
+            raised in case inconsistencies are found.
+            
+            maxcachesize defines the maximum size of the in-memory
+            transaction cache. It defaults to MAXCACHESIZE if not
+            given.
+            
+            XXX Save keysize in storage file header.
+            
+        """
+        basemethod(self, name, keysize=keysize, min_recordsize=min_recordsize,
+                   readonly=readonly, recover=recover,
+                   autocommit=autocommit, validate=validate,
+                   index=BeeIndex.BeeFixedLengthStringIndex,
+                   maxcachesize=maxcachesize)
+
+freeze(BeeFixedLengthStringDict)
+
+###
+
 class BeeStringDictCursor(BeeBaseDictCursor):
 
-    """ Cursor for BeeStringDict instances.
+    """ Cursor for BeeStringDict and BeeFixedLengthStringDict instances.
     """
     def read_key(self):
 
