@@ -11,7 +11,7 @@
     with the number of calibration rounds you wish to apply.
 
     Copyright (c) 2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
-    Copyright (c) 2000-2008, eGenix.com Software GmbH; mailto:info@egenix.com
+    Copyright (c) 2000-2014, eGenix.com Software GmbH; mailto:info@egenix.com
     See the documentation for further information on copyrights,
     or contact the author. All Rights Reserved.
 
@@ -19,7 +19,7 @@
 import socket # This module needs access to sockets !
 import select # This module needs access to select !
 import DateTime
-import re,errno,exceptions,string,time
+import re,errno,exceptions,time
 
 # Enable to produce debugging output
 _debug = 0
@@ -137,7 +137,7 @@ def _connect(host, port, timeout):
     else:
         try:
             host = socket.gethostbyname(hostname)
-        except error:
+        except socket.error:
             # Host not found or some other error
             return None
         else:
@@ -158,8 +158,8 @@ def _connect(host, port, timeout):
             continue
         if rc == errno.EINPROGRESS:
             # In progress... wait for connect
-            r,w,e = select.select([],[s],[],timeout)
-            if w:
+            r,w,e = select.select([s],[],[],timeout)
+            if r:
                 # Connect
                 s.setblocking(1)
                 try:
@@ -199,7 +199,7 @@ def _read(socket, size, timeout,
 
 def _get_daytime(daytime_servers=daytime_servers, timeout=TIMEOUT,
 
-                 strip=string.strip,_parse_datetime=_parse_datetime,
+                 _parse_datetime=_parse_datetime,
                  DateTimeFromMJD=DateTime.DateTimeFromMJD,
                  Time=DateTime.Time,time=time.time):
 
@@ -225,11 +225,11 @@ def _get_daytime(daytime_servers=daytime_servers, timeout=TIMEOUT,
     for host in daytime_servers:
         if _debug: print 'Trying server: %s' % host
         s = _connect(host, DAYTIME_PORT, timeout)
-        if s:
+        if s is not None:
             data = _read(s, 512, timeout)
             if data is None:
                 continue
-            daytime = strip(data)
+            daytime = data.strip()
             timestamp = time()
             s.close()
             if _debug: print ' read %s' % daytime
@@ -370,10 +370,13 @@ def time_offset(iterations=10,
         local = utctime()
         diff = (nist - local).seconds - (time()-timestamp)
         diffs.append(diff)
-    sum = 0
+    sum = 0.0
     for diff in diffs:
         sum = sum + diff
-    return sum / len(diffs)
+    if diffs:
+        return sum / len(diffs)
+    else:
+        return 0.0
 
 def set_calibration(calibration_offset):
 
@@ -429,11 +432,14 @@ def _update_calibration(use_last=_min_diffs,
 
     """
     global calibration,calibrated,calibrating
-    sum = 0
+    sum = 0.0
     diffs = _diffs[-use_last:]
     for diff in diffs:
         sum = sum + diff
-    calibration = sum / len(diffs)
+    if diffs:
+        calibration = sum / len(diffs)
+    else:
+        calibration = 0.0
     _diffs[:] = []
     set_calibration(calibration)
 
