@@ -1,11 +1,11 @@
 """ Namespace objects.
 
     Copyright (c) 1998-2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
-    Copyright (c) 2000-2008, eGenix.com Software GmbH; mailto:info@egenix.com
+    Copyright (c) 2000-2014, eGenix.com Software GmbH; mailto:info@egenix.com
     See the documentation for further information on copyrights,
     or contact the author. All Rights Reserved.
 """
-import string,types
+import types
 import mx.Tools.NewBuiltins
 from mx.Tools import freeze, attributes
 
@@ -161,7 +161,7 @@ class Namespace:
         if not recursive:
             return template % attributes(self)
         dict = attributes(self)
-        while string.find(template, '%(') >= 0:
+        while template.find('%(') >= 0:
             template = template % dict
         return template
 
@@ -360,19 +360,19 @@ class Namespace:
 
     def __repr__(self,
 
-                 join=string.join,id=id):
+                 id=id):
 
         keys = self.__dict__.keys()
         keys.sort()
         return '<%s.%s (%s) at 0x%x>' % (
             self.__class__.__module__,
             self.__class__.__name__,
-            join(keys,','),
+            ','.join(keys),
             id(self))
 
     def __str__(self,
 
-                join=string.join,id=id):
+                id=id):
 
         items = self.__dict__.items()
         items.sort()
@@ -388,7 +388,7 @@ class Namespace:
                 if len(s) > 200:
                     s = s[:200] + '... (truncated, full length = %i)' % len(s)
             l.append('    %-10s %s' % (k + ':', s))
-        return string.join(l,'\n')
+        return '\n'.join(l)
 
 ###
 
