@@ -6,7 +6,7 @@
     See the documentation for further information on copyrights,
     or contact the author. All Rights Reserved.
 """
-import types,string,operator
+import types, operator
 from mx.Tools import freeze
 import mx.Tools.NewBuiltins
 
@@ -75,7 +75,7 @@ class OrderedMapping:
         for key in self.list:
             value = self.dict[key]
             l.append(repr(key)+': '+repr(value))
-        return '{' + string.join(l,', ') + '}'
+        return '{' + ', '.join(l) + '}'
 
     def __cmp__(self, other):
 
