@@ -16,7 +16,7 @@
   MAL_MEM_DEBUG -- enable malloc output (new,cnew,free,...) [mxstdlib.h]
 
   Copyright (c) 2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
-  Copyright (c) 2000-2011, eGenix.com Software GmbH; mailto:info@egenix.com
+  Copyright (c) 2000-2014, eGenix.com Software GmbH; mailto:info@egenix.com
   See the documentation for further copyright information or contact
   the author.
 
@@ -129,9 +129,9 @@ static void mx_Py_INCREF(PyObject *v,
 		      filename,lineno,name);
     else {
 	Py_INCREF(v);;
-	mxDebugPrintf("[%s:%5i] Py_XINCREF( %-8s at 0x%x [%s]); "
+	mxDebugPrintf("[%s:%5i] Py_XINCREF( %-8s at 0x%lx [%s]); "
 		      "new refcount = %i\n",
-		      filename,lineno,name,(int)v,v->ob_type->tp_name,
+		      filename,lineno,name,(long)v,v->ob_type->tp_name,
 		      v->ob_refcnt);
     }
 }
@@ -152,13 +152,13 @@ static void mx_Py_DECREF(PyObject *v,
 	int refcnt = v->ob_refcnt;
 	Py_DECREF(v);
 	if (refcnt <= 1)
-	    mxDebugPrintf("[%s:%5i] Py_XDECREF( %-8s at 0x%x [%s]); "
+	    mxDebugPrintf("[%s:%5i] Py_XDECREF( %-8s at 0x%lx [%s]); "
 			  "object deleted\n",
-			  filename,lineno,name,(int)v,v->ob_type->tp_name);
+			  filename,lineno,name,(long)v,v->ob_type->tp_name);
 	else
-	    mxDebugPrintf("[%s:%5i] Py_XDECREF( %-8s at 0x%x [%s]); "
+	    mxDebugPrintf("[%s:%5i] Py_XDECREF( %-8s at 0x%lx [%s]); "
 			  "new refcount = %i\n",
-			  filename,lineno,name,(int)v,v->ob_type->tp_name,
+			  filename,lineno,name,(long)v,v->ob_type->tp_name,
 			  v->ob_refcnt);
     }
 }
@@ -168,12 +168,14 @@ static void mx_Py_PRINT_REFCOUNT(PyObject *v,
 				 const char *filename,
 				 int lineno) 
 {
+    if (!Py_DebugFlag)
+	return;
     if (!v)
 	mxDebugPrintf("[%s:%5i] Py_PRINT_REFCOUNT( %-8s == NULL );\n",
 		      filename,lineno,name);
     else {
-	mxDebugPrintf("[%s:%5i] Py_PRINT_REFCOUNT( %-8s at 0x%x [%s]) = %i;\n",
-		      filename,lineno,name,(int)v,v->ob_type->tp_name,
+	mxDebugPrintf("[%s:%5i] Py_PRINT_REFCOUNT( %-8s at 0x%lx [%s]) = %i;\n",
+		      filename,lineno,name,(long)v,v->ob_type->tp_name,
 		      v->ob_refcnt);
     }
 }
@@ -186,6 +188,8 @@ static void mx_Py_PRINT_REFCOUNT(PyObject *v,
 # define Py_XINCREF(x) mx_Py_INCREF((PyObject *)x,#x,__FILE__,__LINE__)
 # undef Py_XDECREF
 # define Py_XDECREF(x) mx_Py_DECREF((PyObject *)x,#x,__FILE__,__LINE__)
+# undef Py_CLEAR
+# define Py_CLEAR(x) {Py_XDECREF(x); x=0;}
 # define Py_DELETE(x) {if (x->ob_refcnt > 1) mxDebugPrintf("[%s:%5i] Py_DELETE( "#x" ) WARNING: Refcount = %i > 1\n",__FILE__,__LINE__,(int)x->ob_refcnt);Py_DECREF(x);}
 # define Py_PRINT_REFCOUNT(x) mx_Py_PRINT_REFCOUNT((PyObject *)x,#x,__FILE__,__LINE__)
 #else
@@ -193,7 +197,8 @@ static void mx_Py_PRINT_REFCOUNT(PyObject *v,
 # define Py_PRINT_REFCOUNT(x) 
 #endif
 
-#define Py_DEC_REF(x) {Py_XDECREF(x); x=0;} /* doing this once too often doesn't hurt */
+#define Py_DEC_REF(x) {Py_XDECREF(x); x=0;} /* doing this once too often doesn't hurt;
+					       same as Py_CLEAR() */
 
 /* Unreference a Python object. This is only used in Python debug
    builds and needed to keep track of all allocated references. Use in
@@ -207,6 +212,23 @@ static void mx_Py_PRINT_REFCOUNT(PyObject *v,
 #endif
 #define mxPy_UNREF(x) _Py_DEC_REFTOTAL
 
+/* --- Debug logging ------------------------------------------------------ */
+
+#ifdef MAL_DEBUG
+
+# define Py_PRINT_REPR(message, obj)				\
+    if (Py_DebugFlag) {						\
+        PyObject *_debug_repr = PyObject_Repr(obj);		\
+        mxDebugPrintf(message, PyString_AsString(_debug_repr));	\
+	Py_XDECREF(_debug_repr);				\
+    }
+
+#else
+
+# define Py_PRINT_REPR(message, obj)
+
+#endif
+
 /* --- Argument passing and checking -------------------------------------- */
 
 /* No arguments expected; also use Py_MethodListEntryNoArgs() for this
@@ -417,7 +439,8 @@ static void mx_Py_PRINT_REFCOUNT(PyObject *v,
 #define PyFloat_Compatible(obj)					\
         (PyInstance_Check(obj) 					\
 	 ? PyObject_HasAttrString(obj, "__float__")		\
-	 : (obj->ob_type->tp_as_number->nb_float != NULL))
+	 : (obj->ob_type->tp_as_number != NULL &&		\
+            obj->ob_type->tp_as_number->nb_float != NULL))
 
 /* --- Text macros -------------------------------------------------------- */
 
@@ -521,6 +544,8 @@ static void mx_Py_PRINT_REFCOUNT(PyObject *v,
     Py_Assert(x.tp_basicsize >= (int)sizeof(PyObject),	        \
 	      PyExc_SystemError,				\
 	      "Internal error: tp_basicsize of "#x" too small");\
+    if (PyType_Ready(&x) < 0)					\
+	goto onError;						\
 }
 
 /* Error reporting for module init functions */
