@@ -1,7 +1,7 @@
 """ Tools - Add-ons for Python written in C for performance.
 
     Copyright (c) 2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
-    Copyright (c) 2000-2008, eGenix.com Software GmbH; mailto:info@egenix.com
+    Copyright (c) 2000-2014, eGenix.com Software GmbH; mailto:info@egenix.com
     See the documentation for further information on copyrights,
     or contact the author. All Rights Reserved.
 
@@ -12,7 +12,7 @@ from mxTools import *
 from mxTools import __version__
 
 # Needed Python imports
-import operator,types,string,time,sys,os,re
+import operator,types,time,sys,os,re
 
 #############################################################################
 #
@@ -183,7 +183,7 @@ _hexcode = ("00", "01", "02", "03", "04", "05", "06", "07", "08", "09",
 
 def hexencode(data,
 
-              hexcode=_hexcode,ord=ord,join=string.join):
+              hexcode=_hexcode,ord=ord):
 
     """ HEX encode a data string.
 
@@ -195,7 +195,7 @@ def hexencode(data,
     append = l.append
     for c in data:
         append(hexcode[ord(c)])
-    return join(l,'')
+    return ''.join(l)
 
 
 #############################################################################
@@ -311,7 +311,7 @@ _integerRangeRE = re.compile('\s*(-?\d+)\s*-\s*(-?\d+)\s*$')
 
 def srange(s,
 
-           split=string.split,integer=_integerRE,
+           integer=_integerRE,
            integerRange=_integerRangeRE):
 
     """ Converts a textual representation of integer numbers and ranges
@@ -325,7 +325,7 @@ def srange(s,
     """
     l = []
     append = l.append
-    for entry in split(s,','):
+    for entry in s.split(','):
         m = integer.match(entry)
         if m:
             append(int(m.groups()[0]))
@@ -378,7 +378,7 @@ def splitdomain(hostname=None):
 
     """
     hostname, ip = fqhostname(hostname)
-    l = string.split(hostname, '.', 1)
+    l = hostname.split('.', 1)
     if len(l) == 1:
         return (hostname, '')
     return tuple(l)
@@ -632,19 +632,19 @@ def localized(instance):
                 dict[a] = getattr(instance,a)
     return dict
 
-def ascii2int(str,base=10,
+def ascii2int(str, base=10,
 
-              orig_atoi=string.atoi):
+              int=int):
 
     """ Convert a string to an integer.
 
-        Works like string.atoi except that in case of an error no
+        Works like int() except that in case of an error no
         exception raised but 0 is returned; this makes it useful in
         conjunction with map().
 
     """
     try:
-        return orig_atoi(str,base)
+        return int(str, base)
     except:
         return 0
 
@@ -668,10 +668,10 @@ def str2time(x,
     ti = now[3:5]
     da = now[0:3]
     try:
-        x = string.split(x)
+        x = x.split()
         if len(x) == 0: return -3
-        d = string.splitfields(x[0],'.')
-        t = string.splitfields(x[-1],':')
+        d = x[0].split('.')
+        t = x[-1].split(':')
         if len(t) > 1:
             # Date and time
             if len(t) < 2: return -1
@@ -688,7 +688,7 @@ def str2time(x,
             # Date given
             if len(d) != 3: return -2
             d.reverse()
-            da = tuple(map(string.atoi,d))
+            da = tuple([int(value) for value in d])
             if da[0] == 0:
                 # Year is missing
                 da = (now[0],) + da[1:3]
@@ -726,13 +726,13 @@ def filecontent(filename,default=''):
 
 def long2str(x):
 
-    """ Convert long integer x to a string.
+    """ Convert long integer x to a bytes string.
     """
     l = ( x        & 0xff,
          (x >> 8)  & 0xff,
          (x >> 16) & 0xff,
          (x >> 24) & 0xff)
-    return string.join(map(chr,l))
+    return ''.join([chr(x) for x in l])
     
 ### Hack to enable module finalization
 
@@ -823,7 +823,7 @@ def func_sig(func):
     if code.co_flags & 0x0008: # CO_VARKEYWORDS
         args.append('**'+code.co_varnames[callargs])
         callargs = callargs + 1
-    return '%s(%s)' % (fname,string.join(args,', '))
+    return '%s(%s)' % (fname,', '.join(args))
 
 def func_call(level=1):
 
@@ -872,7 +872,7 @@ def func_call(level=1):
     else:
         where = '# called from <toplevel>'
     del frame,code # you never know...
-    return '%s(%s) %s' % (fname,string.join(l,', '), where)
+    return '%s(%s) %s' % (fname,', '.join(l), where)
 
 def localize_builtins():
 
@@ -996,7 +996,7 @@ code256 = tuple(map(chr,range(256)))
 
 def base(x,b,code=hexcode,
          # Locals:
-         join=string.join,divmod=divmod):
+         divmod=divmod):
     
     """ Return a string representation of integer x in base b.
 
@@ -1011,7 +1011,7 @@ def base(x,b,code=hexcode,
         x,y = divmod(x,b)
         append(code[y])
     l.reverse()
-    return join(l,'')
+    return ''.join(l)
 
 def filedate(path,
 
@@ -1127,9 +1127,7 @@ def _uu_decode(data):
 
     return outfile.getvalue()
 
-def encodedata(data, encoding,
-
-               lower=string.lower):
+def encodedata(data, encoding):
 
     """ Encode data using the given encoding.
 
@@ -1142,7 +1140,7 @@ def encodedata(data, encoding,
         supported natively by Python via the codec registry.
     
     """
-    encoding = lower(encoding)
+    encoding = encoding.lower()
     
     if encoding == 'base64':
         import base64
@@ -1151,24 +1149,11 @@ def encodedata(data, encoding,
     elif encoding == 'hex' or \
          encoding == 'hexlines':
         from mx.TextTools import str2hex
-        import cStringIO
         result = str2hex(data)
         if encoding == 'hexlines':
             return _addlinebreaks(result, 72)
         return result
 
-    elif encoding == 'uu':
-        import binascii
-        
-        out_file.write('begin %o %s\n' % ((mode&0777),name))
-        str = in_file.read(45)
-        while len(str) > 0:
-            out_file.write(binascii.b2a_uu(str))
-            str = in_file.read(45)
-        out_file.write(' \nend\n')
-
-        return base64.encodestring(data)
-    
     else:
         # This works in Python >=2.0 only
         try:
@@ -1176,9 +1161,7 @@ def encodedata(data, encoding,
         except AttributeError:
             raise ValueError, 'unknown encoding "%s"' % encoding
 
-def decodedata(data, encoding,
-
-               lower=string.lower):
+def decodedata(data, encoding):
 
     """ Decode data using the given encoding.
 
@@ -1191,7 +1174,7 @@ def decodedata(data, encoding,
         supported natively by Python via the codec registry.
     
     """
-    encoding = lower(encoding)
+    encoding = encoding.lower()
     
     if encoding == 'base64':
         import base64
@@ -1201,8 +1184,8 @@ def decodedata(data, encoding,
          encoding == 'hexlines':
         from mx.TextTools import hex2str
         # Remove whitespace
-        l = string.split(data)
-        data = string.join(l, '')
+        l = data.split()
+        data = ''.join(l)
         # Decode
         return hex2str(data)
 
